//author: Sandro Wrzalek (AG Nordmeier - TSL)
//Version: v1.8
//Description: Script with a webcamstream and filters for effects linked to an html input.

/* global navigator*/
/* global document*/
/* global window*/
/* global $*/


// GLOBAL VARIABLES

var photo;
var photoData;
//var invertingStream = false;


// GENERAL FUNCTIONS

function duplicateArray(array) {
  "use strict";
  var cloneArray = new Array([array.length]);
  for (var i = 0; i < array.length; i++){
    cloneArray[i] = array[i];
  }

  return cloneArray;
}

// VIDEOFILTERS
/*
function doInvertStream (streamPixelData) {
  for (var i = 0; i < streamPixelData; i += 4) {
    streamPixelData[i] = 255 - streamPixelData[i]; //red
    streamPixelData[i + 1] = 255 - streamPixelData[i + 1]; //green
    streamPixelData[i + 2] = 255 - streamPixelData[i + 2]; //blue

  }
}
*/


function doContrastStream (streamPixelData, cont) {
  "use strict";

  var average;

  for (var i = 0; i < streamPixelData.length; i += 4) {
    average = Math.round( ( streamPixelData[i] + streamPixelData [i + 1] + streamPixelData[i + 2]) / 3 );
    //light colors (everything above 127 up to 255 is white)
    if (average > 127) {
      streamPixelData[i] += cont; //red
      streamPixelData[i + 1] += cont; //green
      streamPixelData[i + 2] += cont; //blue
    }
    //dark colors (everything under 127 down to 0 is black)
    else {
      streamPixelData[i] -= cont; //red
      streamPixelData[i + 1] -= cont; //green
      streamPixelData[i + 2] -= cont; //blue
    }
  }
}


function doBrightnessStream (streamPixelData, bright) {
  "use strict";

  for (var i = 0; i < streamPixelData.length; i += 4) {
    //light colors (everything black = 0 and white = 255)
    streamPixelData[i] += bright; //red
    streamPixelData[i + 1] += bright; //green
    streamPixelData[i + 2] += bright; //blue
  }
}

// VIDEOSTREAM

(function() {
  "use strict";
  var display = document.getElementById("display");
  var displayContext = display.getContext("2d");
  var feed = document.getElementById("feed");
  var feedContext = feed.getContext("2d");
  var video = document.getElementById("webcam");
  var vendorUrl = window.URL || window.webkitURL;

  navigator.getMedia = navigator.getUserMedia ||
                        navigator.mozGetUserMedia ||
                        navigator.webkitGetUserMedia ||
                        navigator.msGetUserMedia ||
                        navigator.oGetUserMedia;

  navigator.getMedia(
    {
      video: true,
      audio: false
    }, function(stream) {
        video.src = vendorUrl.createObjectURL(stream);
        video.play();
    }, function(error) {
        alert("There has been a problem retrieving the streams - did you allow access?");
    });

  function draw (videoStream, context, width, height) {
    context.drawImage(video, 0, 0, width, height);
    setTimeout(draw, 10, videoStream, context, width, height);
  }

  function mirror (feederContext, mirrorContext, width, height) {
    var mirrorData = feederContext.getImageData(0, 0, width, height);
    var data = mirrorData.data;

    //INVERTING PIXELS
    /*
    if (invertingStream) {
      doInvertStream(data);
    }
    */

    //CONTRAST
    var cont = getContrast();
    doContrastStream(data, cont);


    //BRIGHTNESS
    var bright = getBrightness();
    doBrightnessStream(data, bright);

    mirrorContext.putImageData(mirrorData, 0, 0);

    setTimeout(mirror, 50, feederContext, mirrorContext, width, height);
  }


  display.width = video.clientWidth;
  display.height = video.clientHeight;
  feed.width = video.clientWidth;
  feed.height = video.clientHeight;

  video.addEventListener("play", function() {
    draw(this, feedContext, feed.width, feed.height);
    mirror(feedContext, displayContext, display.width, display.height);
  },
  false);

})();



// IMAGEFUNCTIONS

function takePhoto() {
  "use strict";
  photo = document.getElementById("photo");
  var photoContext = photo.getContext("2d");

  var feed = document.getElementById("display");
  var feedContext = feed.getContext("2d");
  var feedData = feedContext.getImageData(0, 0, feed.width, feed.height);

  photo.width = feed.width;
  photo.height = feed.height;

  photoContext.putImageData(feedData, 0, 0);
}

function backupImageData(image) {
  "use strict";
  var imageContext = image.getContext("2d");
  var imageData = imageContext.getImageData(0, 0, image.width, image.height);
  var backupTemp = imageData.data;

  var backup = duplicateArray(backupTemp);

  return backup;
}

//filters


function doInvertPixels (image, imageDataArray) {
  "use strict";
  var imageContext = image.getContext("2d");
  var imageData = imageContext.getImageData(0, 0, image.width, image.height);
  var data = imageData.data;

  for (var i = 0; i < imageDataArray.length; i += 4) {
    imageDataArray[i] = 255 - imageDataArray[i]; //red
    imageDataArray[i + 1] = 255 - imageDataArray[i + 1]; //green
    imageDataArray[i + 2] = 255 - imageDataArray[i + 2]; //blue

  }

  for (var j = 0; j < data.length; j += 1) {
    data[j] = imageDataArray[j];
  }

  imageContext.putImageData(imageData, 0, 0);


}


function opacity (opac, image, imageDataArray) {
  "use strict";

  var imageContext = image.getContext("2d");
  var imageData = imageContext.getImageData(0, 0, image.width, image.height);
  var data = imageData.data;

  for (var i = 3; i < imageDataArray.length; i += 4) {
    imageDataArray[i] = opac;  //transparency
  }

  for (var j = 0; j < data.length; j += 1) {
    data[j] = imageDataArray[j];
  }

  //Debug
  //console.log(photoData);


  imageContext.putImageData(imageData, 0, 0);

}

function contrast (cont, image, imageDataArray) {
  //for safeteyreasons maximum value for contrast is 127, but its also pissible to catch negative values or values which are above 255
  "use strict";
  var imageContext = image.getContext("2d");
  var imageData = imageContext.getImageData(0, 0, image.width, image.height);
  var data = imageData.data;

  var average;

  for (var i = 0; i < imageDataArray.length; i += 4) {
    average = Math.round( ( imageDataArray[i] + imageDataArray [i + 1] + imageDataArray[i + 2]) / 3 );
    //light colors (everything above 127 up to 255 is white)
    if (average > 127) {
      imageDataArray[i] += cont; //red
      imageDataArray[i + 1] += cont; //green
      imageDataArray[i + 2] += cont; //blue
    }
    //dark colors (everything under 127 down to 0 is black)
    else {
      imageDataArray[i] -= cont; //red
      imageDataArray[i + 1] -= cont; //green
      imageDataArray[i + 2] -= cont; //blue
    }
  }

  for (var j = 0; j < data.length; j += 1) {
    data[j] = imageDataArray[j];
  }

  //Debug
  //console.log(photoData);
  //console.log(data);

  imageContext.putImageData(imageData, 0, 0);

}

function brightness (bright, image, imageDataArray) {
  //for safeteyreasons maximum value for contrast is 127, but its also pissible to catch negative values or values which are above 255
  "use strict";
  var imageContext = image.getContext("2d");
  var imageData = imageContext.getImageData(0, 0, image.width, image.height);
  var data = imageData.data;

  for (var i = 0; i < imageDataArray.length; i += 4) {
    //light colors (everything black = 0 and white = 255)
    imageDataArray[i] += bright; //red
    imageDataArray[i + 1] += bright; //green
    imageDataArray[i + 2] += bright; //blue
  }

  for (var j = 0; j < data.length; j += 1) {
    data[j] = imageDataArray[j];
  }

  //Debug
  //console.log(photoData);
  //console.log(data);

  imageContext.putImageData(imageData, 0, 0);

}
//SLIDERS

var slider = ["sliderButtonOpacity", "sliderBackgroundOpacity", "sliderButtonContrast", "sliderBackgroundContrast", "sliderButtonBrightness", "sliderBackgroundBrightness"];

$("#sliderButtonOpacity").udraggable({
    containment: "parent",
    axis: "x"
});

$("#sliderButtonContrast").udraggable({
    containment: "parent",
    axis: "x"
});

$("#sliderButtonBrightness").udraggable({
    containment: "parent",
    axis: "x"
});

//sliderfunctions

function sliderGetButtonPosition (sliderButton, sliderBackground) {
  "use strict";
  var background = document.getElementById(sliderBackground);
  var styleBackground = window.getComputedStyle(background);
  var widthBackground = styleBackground.getPropertyValue("width");
  widthBackground = parseInt(widthBackground);

  var button = document.getElementById(sliderButton);
  var styleButton = window.getComputedStyle(button);
  var leftButton = styleButton.getPropertyValue("left");
  var widthButton = styleButton.getPropertyValue("width");
  leftButton = parseInt(leftButton);
  widthButton = parseInt(widthButton);


  var relativePosition = (leftButton) / (widthBackground - widthButton);

  return relativePosition;
}

//INPUTS

var numberInputs = ["opacityInput", "contrastInput", "brightnessInput"];

function getNumberInput (inputName) {
  "use strict";
  var input = document.getElementById(inputName);
  var inputValue = input.value;

  return inputValue;

}

function setNumberInput (inputname, number){
  "use strict";

  document.getElementById(inputname).value = number;
}

function scaleOpacity (number) {
  "use strict";
  return number * 100;
}

function scaleContrast (number) {
  "use strict";
  return number * 254;
}

function scaleBrightness (number) {
  "use strict";
  var bright = -50 + number * 100;
  return bright;
}

function showNumber(sliderButton, sliderBackground, inputname, scaleFunction) {
  "use strict";
  var pos = sliderGetButtonPosition(sliderButton, sliderBackground);
  var scaleValue = scaleFunction(pos);
  setNumberInput(inputname, scaleValue);
}

//FEATURES
function getOpacity(){
  "use strict";
  var opac = 256 - 256 * sliderGetButtonPosition(slider[0], slider[1]); //0 means not visible 256 completely visible

  return opac;
}

function getContrast(){
  "use strict";
  var cont = 127 * sliderGetButtonPosition(slider[2], slider[3]);

  return cont;
}

function getBrightness(){
  "use strict";
  var bright = 254 * (sliderGetButtonPosition(slider[4], slider[5]) - 0.5);

  return bright;
}


function setOpacity(){
  "use strict";
  var opac = getOpacity();

  //Debug
  //console.log(opac);

  if (photo !== undefined) {
    opacity(opac, photo, photoData);
    showNumber(slider[0], slider[1], numberInputs[0], scaleOpacity);
  }
  else {
    alert("Bitte erst kalibrieren!");
  }

}

var newContrast = getContrast();
var oldContrast;

function setContrast(){
  "use strict";
  oldContrast = newContrast;

  var cont = getContrast();

  newContrast = cont;

  if (photo !== undefined) {
    cont = newContrast - oldContrast;

    contrast(cont, photo, photoData);
    showNumber(slider[2], slider[3], numberInputs[1], scaleContrast);

    //Debug
    //console.log(cont);
  }
  else {
    alert("Bitte erst kalibrieren!");
  }
}

var newBrightness = getBrightness();
var oldBrightness;

function setBrightness(){
  "use strict";
  oldBrightness = newBrightness;

  var bright = getBrightness();

  newBrightness = bright;

  if (photo !== undefined) {
    bright = newBrightness - oldBrightness;

    brightness(bright, photo, photoData);

    showNumber(slider[4], slider[5], numberInputs[2], scaleBrightness);

    //Debug
    //console.log(cont);
  }
  else {
    alert("Bitte erst kalibrieren!");
  }
}


//Routine

function calibrate(){
  "use strict";
  var opac = getOpacity();

  var cont = getContrast();

  var bright = getBrightness();

  takePhoto();
  photoData = backupImageData(photo);

  //invertingStream = true;
  doInvertPixels(photo, photoData);

  opacity(opac, photo, photoData);
  contrast(cont, photo, photoData);
  brightness(bright, photo, photoData);

  showNumber(slider[0], slider[1], numberInputs[0], scaleOpacity);
  showNumber(slider[2], slider[3], numberInputs[1], scaleContrast);
  showNumber(slider[4], slider[5], numberInputs[2], scaleBrightness);

}


//TO DO


//BUGS
